package DE_EPAGES::OAuthTokenClient::t::ConsumerClient;
use base DE_EPAGES::Database::API::Test;

use DE_EPAGES::OAuthTokenClient::API::OAuth::ConsumerClient;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use HTTP::Response;
use HTTP::Status qw ( :constants :is status_message );
use Test::More tests => 11;
use Test::MockObject;

use strict;

sub test {
    my $ExpectedBaseURL = LoadRootObject->get('OAuthServiceBaseUrl') // LoadRootObject->get('EPagesJBaseURL');
    my $Client = DE_EPAGES::OAuthTokenClient::API::OAuth::ConsumerClient->create;
    is($Client->rsBaseUrl(), $ExpectedBaseURL, 'rsBaseUrl');
    $Client->{BaseURL} = 'http://localhost:8088/rs-test/';
    $Client->{'HttpClient'} = _mockHTTPClient( {
        'post'   => \&_assertCreateConsumerContent,
        'delete' => \&_assertDeleteConsumerContent,
        'get'    => \&_assertGetConsumerContent,
    } );

    # check urls
    is($Client->_consumerPath('testConsumer'), 'consumers/testConsumer', 'single consumer path ok'); ## no critic qw(ProhibitPrivateSubroutineUse)
    is($Client->_consumerPath('key test'), 'consumers/key%20test', 'consumer path (urlencoded) ok'); ## no critic qw(ProhibitPrivateSubroutineUse)

    #  create consumer
    my $Consumer = $Client->createConsumer('testConsumer');
    is($Consumer->key, 'b177d4939a934974adf3560e8b50a0d9', 'Consumer is set to: '. $Consumer->key);
    is($Consumer->secret, '181fddd76ddf4fe48277d43ee7d648e4', 'Consumer secret is set to: '. $Consumer->secret);

    # delete consumer
    ok($Client->deleteConsumer('testConsumer'), 'delete consumer successful');

    # get consumer
    $Consumer = $Client->getConsumer('testConsumer');
    is($Consumer->name, 'testConsumer', 'consumer key');
    is($Consumer->key, 'b177d4939a934974adf3560e8b50a0d9', 'consumer token');
    is($Consumer->secret, '181fddd76ddf4fe48277d43ee7d648e4', 'consumer secret');
}

sub _mockHTTPClient {
    my ($hMockMethods) = @_;
    my $mockRemote     = Test::MockObject->new();
    $mockRemote->set_isa('DE_EPAGES::EPagesJ::API::Object::HTTPClient');
    $mockRemote->fake_module( 'DE_EPAGES::EPagesJ::API::Object::HTTPClient', %$hMockMethods );
    while ( my ( $method, $handler ) = each %$hMockMethods ) {
        $mockRemote->mock( $method, $handler );
    }
    return $mockRemote;
}

sub _assertGetConsumerContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    return HTTP::Response->new( HTTP_CREATED, status_message(HTTP_CREATED), undef,
        '{"key":"b177d4939a934974adf3560e8b50a0d9","secret":"181fddd76ddf4fe48277d43ee7d648e4","name":"testConsumer"}'
    );
}

sub _assertCreateConsumerContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/consumers?name=testConsumer', 'post path for create' );
    return HTTP::Response->new( HTTP_CREATED, status_message(HTTP_CREATED), undef,
        '{"key":"b177d4939a934974adf3560e8b50a0d9","secret":"181fddd76ddf4fe48277d43ee7d648e4","name":"testConsumer"}'
    );
}

sub _assertDeleteConsumerContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/consumers/testConsumer', 'post path for delete' );
    return HTTP::Response->new( HTTP_NO_CONTENT, status_message(HTTP_NO_CONTENT), undef, undef );
}

__PACKAGE__->runScript;
