package DE_EPAGES::Product::t::ProductResource;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Test::More tests => 9;
use Test::Exception;

use Data::Dumper;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Product::API::Resource::ProductCollection;

## no critic qw( ProhibitMagicNumbers )

sub test {
    my $self = shift;

    $self->test_GetByID;
    $self->test_GetAll;
}

sub test_GetByID {
    my $self = shift;

    my $LocaleID = 'en_GB';
    my $CurrencyID = 'EUR';

    my $Resource = DE_EPAGES::Product::API::Resource::ProductCollection->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my @GUIDs = (
        $ProductFolder->child('MyPetrol')->guid,
        'non-existing-guid',
        $ProductFolder->child('MyJacket')->guid,
        $ProductFolder->child('MyJacket')->child('SubProducts')->child('JacketBlack5')->guid, # not visible!
        $ProductFolder->child('MyJacket')->child('SubProducts')->child('JacketBlack2')->guid,
    );

    my $hData = $Resource->get( $self->shop, $LocaleID, $CurrencyID, \@GUIDs );
    my $ahProducts = delete $hData->{'items'};
    is_deeply( $hData, {
        results => 3,
        page => 1,
        resultsPerPage => 10,
    }, 'result.items') or diag Dumper($hData, $ahProducts);

    is scalar @$ahProducts, 3, 'number of items';
    is $ahProducts->[0]->{'productId'}, $ProductFolder->child('MyPetrol')->guid, 'items[0]';
    is $ahProducts->[1]->{'productId'}, $ProductFolder->child('MyJacket')->guid, 'items[1]';
    is $ahProducts->[2]->{'productId'}, $ProductFolder->child('MyJacket')->child('SubProducts')->child('JacketBlack2')->guid, 'items[2]';

    my $hProduct = $ahProducts->[0];
    my $hPrice = $hProduct->{'priceInfo'}->{'price'};
    is $hPrice->{'locale'}, $LocaleID, 'locale';
    is $hPrice->{'currency'}, $CurrencyID, 'currency';

    throws_ok sub { $Resource->get( $self->shop, $LocaleID, $CurrencyID, [1..11] ) }, qr/BadRequest/, 'more than 10 ids';
}

sub test_GetAll {
    my $self = shift;

    my $Resource = DE_EPAGES::Product::API::Resource::ProductCollection->new;
    my $hData = $Resource->get( $self->shop, undef, undef, [] );
    is scalar(@{$hData->{'items'}}), 2, 'get all products';
}

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';
    $self->shop->set({
        DomainName => 'test-domain',
        HasSSLCertificate => 1
    });

    note 'importing import.xml';
    ImportObjects('import.xml', $self->shop->parent->pathString);
}

__PACKAGE__->runScript;
